import { ObserverName, DurationBreakdown } from './general';
import { ViewRenderCommitment } from './view-render-commitment';
/**
 * Represents the context to be fed into a view rendered by an {@link OnObserverBaseDirective `*onObserver`} directive.
 * The context is immutable.
 *
 * @export
 * @class OnObserverContext
 * @template TResolved The type of value emitted by the observable the directive is observing.
 */
export declare class OnObserverContext<TResolved> {
    /** The index of the view rendered by the directive. If the directive is in `'single'` view mode, this will always be 0. */
    readonly index: number;
    /** The name of the observer call which triggered this context creation. */
    readonly call: ObserverName;
    /**
     * (Optional) The time left for the view to be rendered. Only used when {@link OnObserverBaseDirective.showFor `showFor`}
     * is specified in the directive.
     */
    readonly remaining?: DurationBreakdown | undefined;
    /**
     * @deprecated Use {@link OnObserverContext.remaining `remaining`} instead. This will be removed in v6.0.0.
     */
    readonly showingFor?: DurationBreakdown | undefined;
    /**
     * (Optional) The time elapsed from the moment the view was rendered. Only used when {@link OnObserverBaseDirective.showFor `showFor`}
     * is specified in the directive.
     */
    readonly elapsed?: DurationBreakdown | undefined;
    [key: string]: unknown;
    /**
     * The resolved value as emitted by the original observable.
     * This allows assigning the emitted value to a variable using `let varName;`.
     *
     * @type {TResolved}
     */
    readonly $implicit?: TResolved;
    /**
     *  Creates an instance of OnObserverContext.
     */
    constructor(
    /**
     * The selector of the directive which is creating this context. This will be used to assign the emitted value to a
     * property matching the selector, thus enabling the use of the microsyntax `as` keyword.
     */
    selector: string, 
    /** The index of the view rendered by the directive. If the directive is in `'single'` view mode, this will always be 0. */
    index: number, 
    /** The name of the observer call which triggered this context creation. */
    call: ObserverName, 
    /** (Optional) The value, if any, that was emitted by the original observable. */
    value?: TResolved, 
    /**
     * (Optional) The time left for the view to be rendered. Only used when {@link OnObserverBaseDirective.showFor `showFor`}
     * is specified in the directive.
     */
    remaining?: DurationBreakdown | undefined, 
    /**
     * @deprecated Use {@link OnObserverContext.remaining `remaining`} instead. This will be removed in v6.0.0.
     */
    showingFor?: DurationBreakdown | undefined, 
    /**
     * (Optional) The time elapsed from the moment the view was rendered. Only used when {@link OnObserverBaseDirective.showFor `showFor`}
     * is specified in the directive.
     */
    elapsed?: DurationBreakdown | undefined);
    /**
     * Creates a context object for the specified view render commitment.
     *
     * @static
     * @template T The type of value emitted by the observable.
     * @param {string} onObserverSelector The selector of the directive which is creating this context.
     * @param {number} index The index of the view rendered by the directive.
     * @param {ViewRenderCommitment<T>} commitment The view render commitment from which to create the context.
     * @return {OnObserverContext<T>} A context object for the specified view render commitment.
     */
    static fromCommitment<T>(onObserverSelector: string, index: number, { call: { name, value } }: ViewRenderCommitment<T>): OnObserverContext<T>;
}
